/*!
 * \file ftdi_api.h
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 1.12
 * \date 20140106
 * \brief FTDI APIs
 *
 * Copyright © 2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev 9     - 20131025 - Initial version \n
 * 0.2     - rev       - 2013xxxx - Updated to use D2XX instead of MPSSE\n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.06    - rev 150   - 20131216 - Corrected bitfield write \n
 * 1.12    - rev 165   - 20140106 - Radix bug fixes \n
 * 2.0     - rev       - 201401   - Dynamic Linking \n
 * \todo
 */
#ifndef __FTDI_API_H__
#define __FTDI_API_H__

/* Standard C libraries */
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
/* OS specific libraries */
#ifdef _WIN32
#include<windows.h>
#endif

/* Include D2XX header*/
#include "ftd2xx.h"

/* Include libMPSSE header */
#include "libMPSSE_spi.h"

/******************************************************************************/
/*                              Macro and type defines                        */
/******************************************************************************/
/* Helper macros */
#define APP_CHECK_STATUS(exp) {if(exp!=FT_OK){printf("%s:%d:%s(): status(0x%x) \
!= FT_OK\n",__FILE__, __LINE__, __FUNCTION__,exp);exit(1);}else{;}};
#define CHECK_NULL(exp){if(exp==NULL){printf("%s:%d:%s():  NULL expression \
encountered \n",__FILE__, __LINE__, __FUNCTION__);exit(1);}else{;}};

// The following ifdef block is the standard way of creating macros
// which make exporting from a DLL simpler.  All files within this DLL
// are compiled with the FTDILIB_EXPORTS symbol defined on the command line.
// This symbol should not be defined on any project that uses this DLL.
// This way any other project whose source files include this file see
// FTDILIB_API functions as being imported from a DLL, whereas this DLL
// sees symbols defined with this macro as being exported.

#ifdef FTDILIB_EXPORTS
#define FTDILIB_API __declspec(dllexport)
#else
#define FTDILIB_API __declspec(dllimport)
#endif


#ifdef __cplusplus
extern "C" {
#endif

/*!
 * \brief FTDI interface initialisation
 *
 * This function initialises FTDI interface with the following parameters
 *
 * \param none
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_Init(void);
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_Init(void);

/*!
 * \brief FTDI interface configuration
 *
 * This function initialises FTDI interface with the following parameters
 *
 * \param[out] *channelConf Configuration of channel
 * \param[in] clock_rate Clock rate of SPI interface
 * \param[in] latency    Latency in ms, typically 1ms for FT232H/FT2232H
 * \param[in] spi_mode   SPI mode 0 to 3. SPI mode defined by (CPOL,CPHA)
 * \param[in] cs_pin     SS pin on DBUSx
 * \param[in] ss_default SS default value
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_Config(25000000,1,3,3,1); // 25MHz, 1ms latency, SPI mode 3, SS=DBUS3 SS default high
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_Config(ChannelConfig* channelConf, uint32 clock_rate, uint8 latency, int spi_mode, int cs_pin, int ss_default);

/*!
 * \brief FTDI destructor.
 *
 * This function is the destructor of FTDI_Init()
 *
 * \param none
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_Free();
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_Free(void);

/*!
 * \brief Prints FTDI channel information
 *
 * This function prints FTDI channel information to screen for user to select
 * the correct channel to communicate with
 *
 * \param none
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_PrintChannelInfo();
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_PrintChannelInfo(void);

/*!
 * \brief FTDI open channel
 *
 * This function opens a channel for communications
 * Only one channel should be opened at a time with this library
 *
 * \param[out] ftHandle   The handle of the FT device
 * \param[in] channel_id The ID of the channel to open
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_OpenChannel(1); // opens channel 1 for communication
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_OpenChannel(FT_HANDLE* ftHandle, uint32 channel_id);

/*!
 * \brief Initialise opened channel
 *
 * This function initialises the channel opened by FTDI_OpenChannel()
 * This function should be called immediately after FTDI_OpenChannel()
 * and before any other channel operation takes place
 *
 * \param[in] ftHandle    The handle of the FT device
 * \param[in] *channelConf Configuration of the channel
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_InitChannel();
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_InitChannel(FT_HANDLE ftHandle, ChannelConfig* channelConf);

/*!
 * \brief Closes FTDI channel
 *
 * This function closes the channel previously opened by FTDI_OpenChannel()
 * Should always be uses in pairs with FTDI_OpenChannel() to release unused resources
 *
 * \param[in] ftHandle   The handle of the FT device
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_CloseChannel();
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_CloseChannel(FT_HANDLE ftHandle);

/*!
 * \brief Read data stream from FTDI
 *
 * This function reads a data stream from FTDI
 *
 * \param[in] ftHandle   The handle of the FT device
 * \param[out] *buffer   Buffer to store received data
 * \param[in]  bytes     Number of bytes to be read
 * \return Returns the size transferred
 * Example Usage:
 * \code
 * uint8 *buffer;
 * buffer = (uint8 *) malloc(25);
 * FTDI_ReadStream(buffer,25);
 * free(buffer);
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_ReadStream(FT_HANDLE ftHandle, uint8* buffer, int bytes);

/*!
 * \brief Write data stream to FTDI
 *
 * This function writes a data stream to FTDI
 *
 * \param[in] ftHandle   The handle of the FT device
 * \param[in] *buffer   Buffer from which data will be read
 * \param[in] bytes     Number of bytes to be read
 * \return Returns the size transferred
 * Example Usage:
 * \code
 * uint8 *buffer;
 * buffer = (uint8 *) malloc(25);
 * FTDI_WriteStream(buffer,25);
 * free(buffer);
 * \endcode
 * \sa
 * \note
 * \warning
 */
FTDILIB_API int FTDI_WriteStream(FT_HANDLE ftHandle, uint8* buffer, int bytes);

/*!
 * \brief Write and read data stream simultaneously
 *
 * This function writes a data stream to FTDI and reads from it simultaneously
 *
 * \param[in] ftHandle   The handle of the FT device
 * \param[in]  *inbuffer   Buffer from which data will be read
 * \param[out] *outbuffer  Buffer to store data to write
 * \param[in]  bytes       Number of bytes
 * \return Returns the size transferred
 * Example Usage:
 * \code
 * uint8 *inbuffer;
 * uint8 *outbuffer;
 * inbuffer = (uint8 *) malloc(25);
 * outbuffer = (uint8 *) malloc(25);
 * FTDI_ReadWrite(inbuffer,outbuffer,25);
 * free(inbuffer);
 * free(outbuffer);
 * \endcode
 * \sa
 * \note Does not work in certain SPI modes
 * \warning DO NOT USE this function without complete testing
 */
FTDILIB_API int FTDI_ReadWrite(FT_HANDLE ftHandle, uint8* inbuffer, uint8* outbuffer, int bytes);

#ifdef __cplusplus
}
#endif

#endif /* __FTDI_API_H__ */