/*!
 * \file ma_api.h
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 2.0
 * \date 20140106
 * \brief MA utility APIs
 *
 * Copyright  2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev 9     - 20131025 - Initial version \n
 * 0.2     - rev 24    - 20131031 - Added write functions \n
 * 0.3     - rev       - 2013xxxx - Modified to use SPI 0\n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 0.95    - rev 132   - 20131211 - Pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.02    - rev 143   - 20131216 - Added auto channel detection \n
 * 1.06    - rev 150   - 20131216 - Corrected bitfield write \n
 * 1.12    - rev 165   - 20140106 - Radix bug fixes \n
 * \todo Add flashing operations
 */

#ifndef __MA_API_H__
#define __MA_API_H__

/* Standard C libraries */
#include<stdio.h>
#include<stdlib.h>
/* OS specific libraries */
#ifdef _WIN32
#include<windows.h>
#endif
#include "ftdi_api.h"
#include "ctype.h"

// The following ifdef block is the standard way of creating macros
// which make exporting from a DLL simpler.  All files within this DLL
// are compiled with the MALIB_EXPORTS symbol defined on the command line.
// This symbol should not be defined on any project that uses this DLL.
// This way any other project whose source files include this file see
// MALIB_API functions as being imported from a DLL, whereas this DLL
// sees symbols defined with this macro as being exported.

#ifdef MALIB_EXPORTS
#define MALIB_API __declspec(dllexport)
#else
#define MALIB_API __declspec(dllimport)
#endif


#define BYTETOBINARYPATTERN "%d%d%d%d%d%d%d%d"
#define BYTETOBINARY(byte)  \
  (byte & 0x80 ? 1 : 0), \
  (byte & 0x40 ? 1 : 0), \
  (byte & 0x20 ? 1 : 0), \
  (byte & 0x10 ? 1 : 0), \
  (byte & 0x08 ? 1 : 0), \
  (byte & 0x04 ? 1 : 0), \
  (byte & 0x02 ? 1 : 0), \
  (byte & 0x01 ? 1 : 0)

#define COMMAND_WRITE (0b0010<<4)
#define COMMAND_READ  (0b0001<<4)

typedef long long int64;

typedef struct
{
    unsigned FI;
    unsigned FLL;
    unsigned FR;
    unsigned FWS;
    unsigned EDF;
} REG0_t;

typedef struct
{
	unsigned R1B0;
	unsigned R1B1;
	unsigned R1B2;
	unsigned R1B3;
	unsigned R1B4;
	unsigned R1B5;
	unsigned R1B6;
	unsigned R1B7;
} REG1_t;

typedef struct
{
	unsigned R2B0;
	unsigned R2B1;
	unsigned R2B2;
	unsigned R2B3;
	unsigned R2B4;
	unsigned R2B5;
	unsigned R2B6;
	unsigned R2B7;
} REG2_t;

typedef struct
{
    unsigned BCT;
} REG3_t;

typedef struct
{
    unsigned ZH;
} REG4_t;

typedef struct
{
    unsigned BIY;
    unsigned BIX;
    unsigned ETY;
    unsigned ETX;
    unsigned ZL;
} REG5_t;

typedef struct
{
	unsigned R6B0;
	unsigned R6B1;
	unsigned R6B2;
	unsigned R6B3;
	unsigned R6B4;
	unsigned R6B5;
	unsigned R6B6;
	unsigned R6B7;
} REG6_t;

typedef struct
{
    unsigned AF;
    unsigned EPT;
    unsigned ABZ;
    unsigned UVW;
    unsigned RD;
    unsigned PWM;
} REG7_t;

typedef struct
{
	unsigned R8B0;
	unsigned R8B1;
	unsigned R8B2;
	unsigned R8B3;
	unsigned R8B4;
	unsigned R8B5;
	unsigned R8B6;
	unsigned R8B7;
} REG8_t;

typedef struct
{
    unsigned F7;
    unsigned F5;
    unsigned F4;
    unsigned F3;
    unsigned F0;
} REG9_t;

typedef struct GenParam_t
{
	uint32 channel;
	int muted;
	int spimode;
	int baudrate;
} GEN_PARAM;

typedef struct RegParam_t
{
	uint8 reg2write[10];
	uint8 reg2write_msk[10];
	uint8 reg2read[10];
	uint8 reg2read_msk[10];
	uint8 reg2read_block[10];
} REG_PARAM;

typedef struct RegVal_t
{
	uint8 regval[10];
} REG_VAL;

typedef unsigned char   uint8;
typedef long long       int64;

#ifdef __cplusplus
extern "C" {
#endif

/*!
 * \brief Generator function for register x
 *
 * This function generates the register value for register x from bit fields
 *
 * \param[in] reg         Register data from bit fields
 * \return Returns register value
 * Example Usage:
 * \code
 * regval = regxgen(regx);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API uint8 reg0gen(REG0_t reg);
MALIB_API uint8 reg1gen(REG1_t reg);
MALIB_API uint8 reg2gen(REG2_t reg);
MALIB_API uint8 reg3gen(REG3_t reg);
MALIB_API uint8 reg4gen(REG4_t reg);
MALIB_API uint8 reg5gen(REG5_t reg);
MALIB_API uint8 reg6gen(REG6_t reg);
MALIB_API uint8 reg7gen(REG7_t reg);
MALIB_API uint8 reg8gen(REG8_t reg);
MALIB_API uint8 reg9gen(REG9_t reg);

/*!
 * \brief MA automatic channel selection
 *
 * This function automatically selects communication channel for MagAlpha EVK
 *
 * \param[out] GenParam         The automatically detected channel is written into parameter setting
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * MA_AutoChannel(GenParam);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_AutoChannel(GEN_PARAM* GenParam);

/*!
 * \brief MA communication initialisation
 *
 * This function initialises MA communication
 *
 * \param[out] ftHandle        The handle of the FT device
 * \param[in]  GenParam         General parameter setting
 * \param[out] *channelConf     Channel configuration
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * MA_Open(ftHandle,GenParam,*channelConf);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_Open(FT_HANDLE* ftHandle, GEN_PARAM GenParam, ChannelConfig* channelConf);

/*!
 * \brief MA communication destructor
 *
 * This function releases MA communication resources
 *
 * \param[in]  ftHandle         The handle of the FT device
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * MA_Close(ftHandle);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_Close(FT_HANDLE ftHandle);

/*!
 * \brief Read sensor data from MA
 *
 * This function reads sensor data from MA
 *
 * \param[in] ftHandle   The handle of the FT device
 * \param[out] *buffer   Pointer to pointer to buffer to store received data
 * \param[in]  bytes     Number of bytes to be read
 * \return Returns the size transferred
 * Example Usage:
 * \code
 * uint8 *buffer;
 * buffer = (uint8 *) malloc(25);
 * MA_ReadData(buffer,25);
 * free(buffer);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_ReadData(FT_HANDLE ftHandle, uint8* buffer, int bytes);

/*!
 * \brief Write register value to MA
 *
 * This function writes register value to MA
 *
 * \param[in]  ftHandle         The handle of the FT device
 * \param[in]  addr    Register address to which data will be written
 * \param[in]  val     Value to write
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FTDI_WriteStream(0,0x05);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_WriteReg(FT_HANDLE ftHandle, int addr, uint8 val);
MALIB_API int MA_WriteReg_Restricted(FT_HANDLE ftHandle, int addr, uint8 val, int auth_key);

/*!
 * \brief Read register value from MA
 *
 * This function reads register value from MA
 *
 * \param[in]  ftHandle         The handle of the FT device
 * \param[in]  addr    Register address from which data will be read
 * \param[out] *val    Pointer to read register value
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * uint8 *regval;
 * FTDI_WriteStream(0,regval);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_ReadReg(FT_HANDLE ftHandle, int addr, uint8* val);

/*!
 * \brief Prints MA register data to screen
 *
 * This function prints register data read from MA to screen with formatting
 *
 * \param[in] data        Data content of the register
 * \param[in] regnum      The register number
 * \param[in] radix       The output radix
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * MA_RegPrint(data, 0, gp_radix);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_RegPrint(uint8 data, int regnum, int radix);

/*!
 * \brief Prints MA data to file
 *
 * This function prints data read from MA to specified file with formatting
 *
 * \param[in] *fp         Pointer to file where the data will be stored
 * \param[in] *data       Pointer to memory block where data is stored
 * \param[in] init_index  The first number of the index
 * \param[in] sample_size Number of samples read from FAS. Each sample has 3 bytes
 * \param[in] format      The output format
 * \param[in] radix       The output radix
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FILE *outputfile;
 * fas_print_file(outputfile,buffer,0,gp_sample_size,gp_output_format,gp_output_radix);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_PrintData(FILE* fp, uint8* data, int init_index, int sample_size, int format, int radix);

/*!
 * \brief Prints MA data error file
 *
 * This function prints discrepencies of data read from MA to specified file with formatting
 *
 * \param[in] *fp         Pointer to file where the data will be stored
 * \param[in] *data       Pointer to memory block where data is stored
 * \param[in] init_index  The first number of the index
 * \param[in] sample_size Number of samples read from FAS. Each sample has 3 bytes
 * \param[in] format      The output format
 * \param[in] radix       The output radix
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * FILE *errorfile;
 * fas_print_file(errorfile,buffer,0,gp_sample_size,gp_output_format,gp_output_radix);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_PrintError(FILE* fp, uint8* data, int init_index, int sample_size, int format, int radix);

/*!
 * \brief MA Library version
 *
 * This function prints register data read from MA to screen with formatting
 *
 * \param[out] **version      Pointer to MAlib version string
 * \return Returns 0 for success
 * Example Usage:
 * \code
 * MA_LibVer(**version);
 * \endcode
 * \sa
 * \note
 * \warning
 */
MALIB_API int MA_LibVer(char version[]);


#ifdef __cplusplus
}
#endif

#endif /* __MA_API_H__ */