/*!
 * \file parser.c
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 1.1
 * \date 20140108
 * \brief Main program argument parser
 *
 * Copyright © 2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev ?     - 20131024 - Initial version \n
 * 0.2     - rev 8     - 20131025 - Added argument parser options \n
 *                                  Updated error report format \n
 * 0.2.1   - rev       - 20131025 - Added include guard \n
 *                                  Improved debug code syntax \n
 *                                  Added doxygen style comments \n
 *                                  Changed filename \n
 * 0.2.2   - rev 23    - 20131030 - Corrected decimal format bugs \n
 * 0.3     - rev 40    - 20131104 - Changed SPI to mode 0 \n
 * 0.4     - rev 49    - 20131104 - Changed program name to "maread" \n
 *                                  Made SPI mode configurable \n
 * 0.5     - rev 64    - 20131112 - Added plain output \n
 * 0.6     - rev 87    - 20131118 - Added baudrate option \n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.02    - rev 143   - 20131216 - Added auto channel detection \n
 * 1.03    - rev 144   - 20121216 - Modified debug behaviour \n
 * 1.04    - rev 146   - 20121216 - Updated help messages \n
 * 1.05    - rev 148   - 20121216 - Default behaviour defined \n
 * 1.10    - rev 165   - 20140108 - Radix bug fixes \n
 * 1.9.3   - rev 210   - 20140129 - 2.0 Pre-release \n
 * \todo
 */
#include "parser.h"


int arg_parser(GEN_PARAM* GenParam, PARSER_PARAM* ParserParam, int argc, char *argv[])
{
    int arg_parsed = 0;
    int exit_now = 0;
    char *str1;
	char malib_ver[30];
    unsigned char completeVersion[] =
    {
        VERSION_MAJOR_INIT,
        '.',
        VERSION_MINOR_INIT,
        '.',
        VERSION_PATCHLEVEL_INIT,
        '.',
        BUILD_YEAR_CH2, BUILD_YEAR_CH3,
        BUILD_MONTH_CH0, BUILD_MONTH_CH1,
        BUILD_DAY_CH0, BUILD_DAY_CH1,
        '.',
        BUILD_HOUR_CH0, BUILD_HOUR_CH1,
        BUILD_MIN_CH0, BUILD_MIN_CH1,
        BUILD_SEC_CH0, BUILD_SEC_CH1,
        '\0'
    };
	GenParam->channel = 1;
	GenParam->muted = 1;
	GenParam->spimode = 0;
	GenParam->baudrate = 25000000;
	ParserParam->data_in = 0;
	
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Program name: %s\n",__FILE__, __LINE__, __FUNCTION__, argv[0]););
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Number of arguments: %d\n",__FILE__, __LINE__, __FUNCTION__, argc););

    // set default behaviour
    if (argc ==1)
    {
        ParserParam->data_in = 0;
    }


    arg_parsed++;
    while ( (argc > arg_parsed) && (!exit_now))
    {
        if ( (argv[arg_parsed][0] == '-') || (argv[arg_parsed][0] == '/'))
        {
            switch (argv[arg_parsed][1])
            {
                case '-':
                    if (!strcmp(argv[arg_parsed],"--help"))
                        argv[arg_parsed] = "-h";
                    else if (!strcmp(argv[arg_parsed],"--version"))
                        argv[arg_parsed] = "-v";
                    else if (!strcmp(argv[arg_parsed],"--file"))
                        argv[arg_parsed] = "-f";
                    else if (!strcmp(argv[arg_parsed],"--output-text"))
                        argv[arg_parsed] = "-ot";
                    else if (!strcmp(argv[arg_parsed],"--output-csv"))
                        argv[arg_parsed] = "-oc";
                    else if (!strcmp(argv[arg_parsed],"--output-plain"))
                        argv[arg_parsed] = "-op";
                    else if (!strcmp(argv[arg_parsed],"--radix-bin"))
                        argv[arg_parsed] = "-xb";
                    else if (!strcmp(argv[arg_parsed],"--radix-dec"))
                        argv[arg_parsed] = "-xd";
                    else if (!strcmp(argv[arg_parsed],"--radix-hex"))
                        argv[arg_parsed] = "-xh";
                    else if (!strcmp(argv[arg_parsed],"--radix-bin-prefix"))
                        argv[arg_parsed] = "-x0b";
                    else if (!strcmp(argv[arg_parsed],"--radix-hex-prefix"))
                        argv[arg_parsed] = "-x0x";
                    else if (!strcmp(argv[arg_parsed],"--list"))
                        argv[arg_parsed] = "-l";
                    else if (!strcmp(argv[arg_parsed],"--report"))
                        argv[arg_parsed] = "-e";
                    else if (!strcmp(argv[arg_parsed],"--append"))
                        argv[arg_parsed] = "-a";
                    else
                    {
                        printf("Syntax error: Invalid argument: %s. For help type:\n%s --help\n", argv[arg_parsed++],argv[0]);
                        #ifndef DEBUG
                            exit(1);
                        #endif
                    }
                    break;
                case 'h':
                    printf( "Usage: %s [options] ...\n"\
                            "-a [--append]\t\t\tAppends output to the end of specified file\n"\
                            "-c{CHANNEL} \t\t\tSets channel to CHANNEL\n"\
                            "-e [--report]\t\t\tEnables report output to FILENAME.info\n"\
                            "-f {FILENAME} \t\t\tOutputs to file FILENAME\n"\
                            "[--file FILENAME]\n"\
                            "-l [--list]\t\t\tLists channel information\n"\
                            "-o {FORMAT} \t\t\tSets output format to FORMAT. FORMAT=text,csv\n"\
                            "  -ot [--output-text]\t\t  Sets output format to text\n"\
                            "  -oc [--output-csv]\t\t  Sets output format to csv\n"\
                            "-x {RADIX} \t\t\tSets radix to RADIX. RADIX=bin,dec,hex,0b,0x\n"\
                            "  -xb [--radix-bin]\t\t  Sets radix to binary\n"\
                            "  -xd [--radix-dec]\t\t  Sets radix to decimal\n"\
                            "  -xh [--radix-hex]\t\t  Sets radix to hex\n"\
                            "  -x0b [--radix-bin-prefix]\t  Sets radix to binary with '0b' prefix\n"\
                            "  -x0x [--radix-hex-prefix]\t  Sets radix to hex with '0x' prefix\n"\
                            "-s{SIZE} \t\t\tSets sample size to SIZE\n"\
                            "-t{TIME}{UNIT} \t\t\tSets sampling time to TIME UNITs. UNIT=us,ms,s\n"\
                            "-h [--help]\t\t\tDisplays this help\n"\
                            "-v [--version]\t\t\tDisplays version information\n"\
                            "This program is built for "SYSTEM_ARCH" with gcc "__VERSION__"\n"\
                            "Report bugs to <benjamin.zhai@sensima.com>\n",argv[0]);
                    arg_parsed++;
                    exit_now = 1;
                    break;
                case 'v':
					MA_LibVer(malib_ver);
                    printf( "MagAlpha read utility v%s\n"\
                            "==================================================\n"\
                            "MA Utility API version %s\n"\
                            "Copyright (C) "YEARS" Sensima Technology SA.\n"\
                            "All rights reserved.\n"\
                            "This software is provided AS IS with NO warranty.\n",completeVersion,malib_ver);
                    arg_parsed++;
                    exit_now = 1;
                    break;
                case 'p':
                    GenParam->spimode = (int) (strtol(argv[arg_parsed++]+2,NULL,10)%4);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -p: SPI mode is %d\n",GenParam->spimode););
                    break;
                case 'b':
                    GenParam->baudrate = (int) strtol(argv[arg_parsed++]+2,NULL,10);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -b: Baudrate is %d\n",GenParam->baudrate););
                    break;
                case 'c':
                    GenParam->channel = (int) strtol(argv[arg_parsed++]+2,NULL,10);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -c: Channel %d\n",GenParam->channel););
                    break;
                case 'l':
                    GenParam->muted = 0;
                    arg_parsed++;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -l: List channels\n"););
                    break;
				case '.':
                    ParserParam->data_in = (int) strtol(argv[arg_parsed++]+2,NULL,0);
					ParserParam->trnsfr16b = 1;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -.: Data is %d\n",ParserParam->data_in););
					break;
                default:
                    printf("Syntax error: Invalid argument: %s. For help type:\n%s --help\n", argv[arg_parsed++],argv[0]);
                    #ifndef DEBUG
                        exit(1);
                    #endif
                    break;
            }
        }
        else
        {
            printf("Syntax error: Invalid argument: %s. For help type:\n%s --help\n", argv[arg_parsed++],argv[0]);
            #ifndef DEBUG
                exit(1);
            #endif
        }
    }
    if (exit_now)
        exit(0);
    return 0;
}
