/*!
 * \file ftdi_api.c
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 1.12
 * \date 20140106
 * \brief FTDI APIs
 *
 * Copyright © 2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev 9     - 20131025 - Initial version \n
 * 0.2     - rev       - 2013xxxx - Updated to use D2XX instead of MPSSE\n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.06    - rev 150   - 20131216 - Corrected bitfield write \n
 * 1.12    - rev 165   - 20140106 - Radix bug fixes \n
 * 1.9.3   - rev 210   - 20140129 - 2.0 Pre-release \n
 * 2.0     - rev       - 201401   - Dynamic Linking \n
 * \todo
 */
 
#include "build_defs.h"
#include "ftdi_api.h"

/******************************************************************************/
/*                              Global variables                              */
/******************************************************************************/

int FTDI_Init(void)
{
#ifdef _MSC_VER
    Init_libMPSSE();
#endif
	return 0;
}

int FTDI_Config(ChannelConfig* channelConf, uint32 clock_rate, uint8 latency, int spi_mode, int cs_pin, int ss_default)
{
    channelConf->Pin = 0;
    channelConf->configOptions = 0;
    channelConf->ClockRate = clock_rate;
    channelConf->LatencyTimer= latency;
    channelConf->Pin |= (ss_default)? 0x00000808 : 0x00000008;
    channelConf->configOptions |= (ss_default)? SPI_CONFIG_OPTION_CS_ACTIVELOW : 0;
    switch (cs_pin)
    {
        case 3: channelConf->configOptions |= SPI_CONFIG_OPTION_CS_DBUS3; break;
        case 4: channelConf->configOptions |= SPI_CONFIG_OPTION_CS_DBUS4; break;
        case 5: channelConf->configOptions |= SPI_CONFIG_OPTION_CS_DBUS5; break;
        case 6: channelConf->configOptions |= SPI_CONFIG_OPTION_CS_DBUS6; break;
        case 7: channelConf->configOptions |= SPI_CONFIG_OPTION_CS_DBUS7; break;
    }
    switch (spi_mode)
    {
        case 0:
            channelConf->configOptions |= SPI_CONFIG_OPTION_MODE0;
            channelConf->Pin |= 0x00000203;
            break;
        case 1:
            channelConf->configOptions |= SPI_CONFIG_OPTION_MODE1;
            channelConf->Pin |= 0x00000203;
            break;
        case 2:
            channelConf->configOptions |= SPI_CONFIG_OPTION_MODE2;
            channelConf->Pin |= 0x00000303;
            break;
        case 3:
            channelConf->configOptions |= SPI_CONFIG_OPTION_MODE3;
            channelConf->Pin |= 0x00000303;
            break;
    }
    
    DEBUG_CODE(fprintf(stderr, "***DEBUG: %s:%d:%s(): SPI Configuration***\n"\
                "***Clock rate = %d\n"\
                "***Latency = %d\n"\
                "***Options = 0x%08x\n"\
                "***Pins = 0x%08x\n",\
                __FILE__, __LINE__, __FUNCTION__,channelConf->ClockRate,channelConf->LatencyTimer,channelConf->configOptions,channelConf->Pin););

    return 0;
}

int FTDI_Free(void)
{
#ifdef _MSC_VER
    Cleanup_libMPSSE();
#endif
    return 0;
}

int FTDI_PrintChannelInfo(void)
{
    FT_STATUS status;
    FT_DEVICE_LIST_INFO_NODE devList;
    int i;
	uint32 channels;

    status = SPI_GetNumChannels(&channels);
    APP_CHECK_STATUS(status);
    printf("Number of available SPI channels = %d\n",channels);

    if(channels>0)
    {
        for(i=0;i<channels;i++)
        {
            status = SPI_GetChannelInfo(i,&devList);
            APP_CHECK_STATUS(status);
            printf("Information on channel number %d:\n",i);
            /* print the dev info */
            printf("        Flags=0x%x\n",devList.Flags);
            printf("        Type=0x%x\n",devList.Type);
            printf("        ID=0x%x\n",devList.ID);
            printf("        LocId=0x%x\n",devList.LocId);
            printf("        SerialNumber=%s\n",devList.SerialNumber);
            printf("        Description=%s\n",devList.Description);
            printf("        ftHandle=0x%x\n",devList.ftHandle); /*is 0 unless open*/
        }
    }
    return 0;
}

int FTDI_OpenChannel(FT_HANDLE* ftHandle, uint32 channel_id)
{
    FT_STATUS status;
    status = SPI_OpenChannel(channel_id,ftHandle);
    APP_CHECK_STATUS(status);
    return 0;
}

int FTDI_InitChannel(FT_HANDLE ftHandle, ChannelConfig* channelConf)
{
    FT_STATUS status;
    status = SPI_InitChannel(ftHandle,channelConf);
    APP_CHECK_STATUS(status);
    return 0;
}

int FTDI_CloseChannel(FT_HANDLE ftHandle)
{
    FT_STATUS status;
    status = SPI_CloseChannel(ftHandle);
    APP_CHECK_STATUS(status);
    return 0;
}

int FTDI_ReadStream(FT_HANDLE ftHandle, uint8 *buffer, int bytes)
{
    FT_STATUS status;
    uint32 sizeTransfered;
    status = SPI_Read(ftHandle, buffer, bytes, &sizeTransfered,
    SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES|SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE|SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
    APP_CHECK_STATUS(status);
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred: %d, Status:%d\n",__FILE__, __LINE__, __FUNCTION__, sizeTransfered,status););
    return (int)sizeTransfered;
}

int FTDI_WriteStream(FT_HANDLE ftHandle, uint8 *buffer, int bytes)
{
    FT_STATUS status;
    uint32 sizeTransfered;
    status = SPI_Write(ftHandle, buffer, bytes, &sizeTransfered,
        SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES|SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE|SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
    APP_CHECK_STATUS(status);
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred: %d, Status:%d\n",__FILE__, __LINE__, __FUNCTION__, sizeTransfered,status););
    return (int)sizeTransfered;
}

int FTDI_ReadWrite(FT_HANDLE ftHandle, uint8 *inbuffer, uint8 *outbuffer, int bytes)
{
    FT_STATUS status;
    uint32 sizeTransfered;
    status = SPI_ReadWrite(ftHandle, inbuffer, outbuffer, bytes, &sizeTransfered,
        SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES|SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE|SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
    APP_CHECK_STATUS(status);
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred: %d, Status:%d\n",__FILE__, __LINE__, __FUNCTION__, sizeTransfered,status););
    return sizeTransfered;
}

