/*!
 * \file ma_api.c
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 2.0
 * \date 20140106
 * \brief MA utility APIs
 *
 * Copyright © 2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev 9     - 20131025 - Initial version \n
 * 0.2     - rev 24    - 20131031 - Added write functions \n
 * 0.3     - rev       - 2013xxxx - Modified to use SPI 0\n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 0.95    - rev 132   - 20131211 - Pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.02    - rev 143   - 20131216 - Added auto channel detection \n
 * 1.06    - rev 150   - 20131216 - Corrected bitfield write \n
 * 1.12    - rev 165   - 20140106 - Radix bug fixes \n
 * 1.9.3   - rev 210   - 20140129 - 2.0 Pre-release \n
 * 2.0     - rev       - 201401   - Dynamic Linking \n
 * \todo Add flashing operations
 */

#include "build_defs.h"
#include "ma_api.h"

// REG0_t reg0={0};
// REG3_t reg3={0};
// REG4_t reg4={0};
// REG5_t reg5={0};
// REG7_t reg7={0};
// REG9_t reg9={0};


// Reg gen functions for Endianness independent implementation

uint8 reg0gen(REG0_t reg)
{
    return((uint8)( ((reg.FI&0x07)<<5)|((reg.FLL&0x03)<<3)|((reg.FR&0x01)<<2)|((reg.FWS&0x01)<<1)|(reg.EDF&0x01)) );
}

uint8 reg1gen(REG1_t reg)
{
    return((uint8)( ((reg.R1B7&0x01)<<7)|((reg.R1B6&0x01)<<6)|((reg.R1B5&0x01)<<5)|((reg.R1B4&0x01)<<4)|((reg.R1B3&0x01)<<3)|((reg.R1B2&0x01)<<2)|((reg.R1B1&0x01)<<1)|(reg.R1B0&0x01) ));
}

uint8 reg2gen(REG2_t reg)
{
    return((uint8)( ((reg.R2B7&0x01)<<7)|((reg.R2B6&0x01)<<6)|((reg.R2B5&0x01)<<5)|((reg.R2B4&0x01)<<4)|((reg.R2B3&0x01)<<3)|((reg.R2B2&0x01)<<2)|((reg.R2B1&0x01)<<1)|(reg.R2B0&0x01) ));
}

uint8 reg3gen(REG3_t reg)
{
    return((uint8)( reg.BCT&0xff ));
}

uint8 reg4gen(REG4_t reg)
{
    return((uint8)( reg.ZH&0xff ));
}

uint8 reg5gen(REG5_t reg)
{
    return((uint8)( ((reg.BIY&0x01)<<7)|((reg.BIX&0x01)<<6)|((reg.ETY&0x01)<<5)|((reg.ETX&0x01)<<4)|(reg.ZL&0x0f) ));
}

uint8 reg6gen(REG6_t reg)
{
    return((uint8)( ((reg.R6B7&0x01)<<7)|((reg.R6B6&0x01)<<6)|((reg.R6B5&0x01)<<5)|((reg.R6B4&0x01)<<4)|((reg.R6B3&0x01)<<3)|((reg.R6B2&0x01)<<2)|((reg.R6B1&0x01)<<1)|(reg.R6B0&0x01) ));
}

uint8 reg7gen(REG7_t reg)
{
    return((uint8)( ((reg.AF&0x03)<<6)|((reg.EPT&0x03)<<4)|((reg.ABZ&0x01)<<3)|((reg.UVW&0x01)<<2)|((reg.RD&0x01)<<1)|(reg.PWM&0x01) ));
}

uint8 reg8gen(REG8_t reg)
{
    return((uint8)( ((reg.R8B7&0x01)<<7)|((reg.R8B6&0x01)<<6)|((reg.R8B5&0x01)<<5)|((reg.R8B4&0x01)<<4)|((reg.R8B3&0x01)<<3)|((reg.R8B2&0x01)<<2)|((reg.R8B1&0x01)<<1)|(reg.R8B0&0x01) ));
}

uint8 reg9gen(REG9_t reg)
{
    return((uint8)( ((reg.F7&0x01)<<7)|((reg.F5&0x01)<<5)|((reg.F4&0x01)<<4)|((reg.F3&0x01)<<3)|(reg.F0&0x01) ));
}

// Automatic channel detection

int MA_AutoChannel(GEN_PARAM* GenParam)
{
    FT_STATUS status;
    FT_DEVICE_LIST_INFO_NODE devList;
    uint32 i;
	uint32 channels;

    status = SPI_GetNumChannels(&channels);
    APP_CHECK_STATUS(status);
    DEBUG_CODE(fprintf(stderr,"****DEBUG: %s:%d:%s(): Number of available SPI channels = %d\n",__FILE__, __LINE__, __FUNCTION__,channels););

    if(channels>0)
    {
        for(i=0;i<channels;i++)
        {
            status = SPI_GetChannelInfo(i,&devList);
            APP_CHECK_STATUS(status);

            if ((devList.ID==0x4036010) && (strstr(devList.SerialNumber,"B")!=NULL))
            {
                GenParam->channel = i;
                DEBUG_CODE(fprintf(stderr,"****DEBUG: %s:%d:%s(): Channel set to = %d\n",__FILE__, __LINE__, __FUNCTION__,GenParam->channel););
				return 0;
            }
        }
    }
    return 1; // unsuccessful
}

// FTDI init operations
int MA_Open(FT_HANDLE* ftHandle, GEN_PARAM GenParam, ChannelConfig* channelConf)
{

    // Initialise SPI
	FTDI_Init();
    FTDI_Config(channelConf,GenParam.baudrate,1,GenParam.spimode,3,1); // baudrate, 1ms latency, SPI mode, SS=DBUS3, SS default high

    // Print channel info and initialise channel
    if(!GenParam.muted)
        FTDI_PrintChannelInfo();
    FTDI_OpenChannel(ftHandle,GenParam.channel);
    if(!GenParam.muted)
        printf("\nChannel opened.\nhandle=0x%x\n",ftHandle);
    FTDI_InitChannel(*ftHandle,channelConf); // Must be called after FTDI_OpenChannel(), before using the channel
    return 0;
}

// FTDI close operations
int MA_Close(FT_HANDLE ftHandle)
{
    FTDI_CloseChannel(ftHandle); // Destructor of FTDI_OpenChannel()
    FTDI_Free(); // Destructor of FTDI_Init()
    return 0;
}

// MA Read Data
int MA_ReadData(FT_HANDLE ftHandle, uint8* buffer, int bytes)
{
	FTDI_ReadStream(ftHandle, buffer, bytes+3);
	memmove(buffer,buffer+3,bytes);
	return 0;
}

// MA initialise
int MA_Cmd(FT_HANDLE ftHandle,int cmd)
{
    int size;
    uint8 buffer[3];
	buffer[0] = (uint8)(cmd&0xff);
	buffer[1] = (uint8)((cmd>>8)&0xff);
	buffer[2] = 0;
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Write+Addr:0x%02x Value:0x%02x\n", __FILE__, __LINE__, __FUNCTION__,buffer[0],buffer[1]););
	size = FTDI_WriteStream(ftHandle, buffer,3);
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred %d.\n",__FILE__, __LINE__, __FUNCTION__,size););
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Write complete.\n",__FILE__, __LINE__, __FUNCTION__););
	return 0;
}

// MA write to register
int MA_WriteReg(FT_HANDLE ftHandle, int addr, uint8 val)
{
    int size;
    uint8 buffer[3];
	if ( (addr==0)||(addr==3)||(addr==4)||(addr==5)||(addr==7)||(addr==9) )
	{
		buffer[0] = COMMAND_WRITE|(addr&0x0f);
		buffer[1] = val;
		buffer[2] = 0;
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Write+Addr:0x%02x Value:0x%02x\n", __FILE__, __LINE__, __FUNCTION__,buffer[0],buffer[1]););
		size = FTDI_WriteStream(ftHandle, buffer,3);
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred %d.\n",__FILE__, __LINE__, __FUNCTION__,size););
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Write complete.\n",__FILE__, __LINE__, __FUNCTION__););
		return 0;
	}
	return 1; // Authentication failed
}

int MA_WriteReg_Restricted(FT_HANDLE ftHandle, int addr, uint8 val, int auth_key)
{
    int size;
    uint8 buffer[3];
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Auth key %x.\n",__FILE__, __LINE__, __FUNCTION__,auth_key););
	if ( (auth_key == 0x5f0762e5)||(addr==0)||(addr==3)||(addr==4)||(addr==5)||(addr==7)||(addr==9) )
	{
		buffer[0] = COMMAND_WRITE|(addr&0x0f);
		buffer[1] = val;
		buffer[2] = 0;
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Write+Addr:0x%02x Value:0x%02x\n",__FILE__, __LINE__, __FUNCTION__, buffer[0],buffer[1]););
		size = FTDI_WriteStream(ftHandle, buffer,3);
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred %d.\n",__FILE__, __LINE__, __FUNCTION__,size););
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Write complete.\n",__FILE__, __LINE__, __FUNCTION__););
		return 0;
	}
	return 1; // Authentication failed
}

// MA read from register
int MA_ReadReg(FT_HANDLE ftHandle, int addr, uint8 *val)
{
    int size,retval;
    uint8 inbuffer[3]={0,0,0};
    uint8 outbuffer[6]={0,0,0,0,0,1};
    inbuffer[0] = COMMAND_READ|(addr&0x0f);
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Read+addr:0x%02x\n",__FILE__, __LINE__, __FUNCTION__, inbuffer[0]););
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Sending READ command.\n",__FILE__, __LINE__, __FUNCTION__););
    size = FTDI_WriteStream(ftHandle,inbuffer,3);
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred %d.\n",__FILE__, __LINE__, __FUNCTION__,size););
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Receiving.\n",__FILE__, __LINE__, __FUNCTION__););
    size = FTDI_ReadStream(ftHandle,outbuffer,6);
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Size transferred %d.\n",__FILE__, __LINE__, __FUNCTION__,size););
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Bytes transferred during register reading:0x%02x 0x%02x 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, outbuffer[3],outbuffer[4],outbuffer[5]););

    if (!outbuffer[5])
    {
        retval = 0;
        *val = outbuffer[4];
    }
    else
        retval = 1;
    return retval;
}

// Data print functions
int MA_RegPrint(uint8 data, int regnum, int radix)
{
    int i;
    if (radix == 0)
    {
            printf( "Register %d: "BYTETOBINARYPATTERN"\n", \
                    regnum,BYTETOBINARY(data));
    }
    else if (radix == 1)
    {
            printf( "Register %d: %d\n", regnum,data);
    }
    else if (radix == 2)
    {
            printf( "Register %d: %02x\n", regnum,data);
    }
    else if (radix == 3)
    {
            printf( "Register %d: 0b"BYTETOBINARYPATTERN"\n", \
                    regnum,BYTETOBINARY(data));
    }
    else if (radix == 4)
    {
            printf( "Register %d: 0x%02x\n", regnum,data);
    }
    return 0;
}


int MA_PrintData(FILE *fp, uint8 *data, int init_index, int sample_size, int format, int radix)
{
    int i;
    if ( (format == 0) && (radix == 0) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "Data %5d: "BYTETOBINARYPATTERN""BYTETOBINARYPATTERN"\tIndex: "BYTETOBINARYPATTERN"\n", \
                    init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
        }
    }
    else if ( (format == 0) && (radix == 1) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "Data %5d: %5d\tIndex: %3d\n", init_index+i, ((unsigned)data[3*i]*256+data[3*i+1]),data[3*i+2]);
        }
    }
    else if ( (format == 0) && (radix == 2) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "Data %5d: %02x%02x\tIndex: %02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
        }
    }
    else if ( (format == 0) && (radix == 3) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "Data %5d: 0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN"\tIndex: 0b"BYTETOBINARYPATTERN"\n", \
                    init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
        }
    }
    else if ( (format == 0) && (radix == 4) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "Data %5d: 0x%02x%02x\tIndex: 0x%02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
        }
    }
    else if ( (format == 1) && (radix == 0) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "%5d,"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN","BYTETOBINARYPATTERN"\n", \
                    init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
        }
    }
    else if ( (format == 1) && (radix == 1) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "%5d,%5d,%3d\n", init_index+i, ((unsigned)data[3*i]*256+data[3*i+1]),data[3*i+2]);
        }
    }
    else if ( (format == 1) && (radix == 2) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "%5d,%02x%02x,%02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
        }
    }
    else if ( (format == 1) && (radix == 3) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "%5d,0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN",0b"BYTETOBINARYPATTERN"\n", \
                    init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
        }
    }
    else if ( (format == 1) && (radix == 4) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp, "%5d,0x%02x%02x,0x%02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
        }
    }
    else if ( (format == 2) && (radix == 0) ) // WARNING: Buffer is not flushed in format 2 when printing to stdout due to speed considerations. 
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp,  BYTETOBINARYPATTERN""BYTETOBINARYPATTERN",", \
                    BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]));
        }
    }
    else if ( (format == 2) && (radix == 1) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp,  "%d,", ((unsigned)data[3*i]*256+data[3*i+1]));
        }
    }
    else if ( (format == 2) && (radix == 2) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp,  "%02x%02x,", data[3*i],data[3*i+1]);
        }
    }
    else if ( (format == 2) && (radix == 3) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp,  "0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN",", \
                    BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]));
        }
    }
    else if ( (format == 2) && (radix == 4) )
    {
        for (i = 0; i < sample_size; i++)
        {
            fprintf(fp,  "0x%02x%02x,", data[3*i],data[3*i+1]);
        }
    }
    return 0;
}

int MA_PrintError(FILE *fp, uint8 *data, int init_index, int sample_size, int format, int radix)
{
    int i;
    if ( (format == 0) && (radix == 0) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "Data %5d: "BYTETOBINARYPATTERN""BYTETOBINARYPATTERN"\tIndex: "BYTETOBINARYPATTERN"\n", \
                        init_index+i-1, BYTETOBINARY(data[3*i-3]), BYTETOBINARY(data[3*i-2]), BYTETOBINARY(data[3*i-1]));
                fprintf(fp, "Data %5d: "BYTETOBINARYPATTERN""BYTETOBINARYPATTERN"\tIndex: "BYTETOBINARYPATTERN"\n", \
                        init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
            }
        }
    }
    else if ( (format == 0) && (radix == 1) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "Data %5d: %5d\tIndex: %3d\n", init_index+i-1, ((unsigned)data[3*i-3]*256+data[3*i-2]),data[3*i-1]);
                fprintf(fp, "Data %5d: %5d\tIndex: %3d\n", init_index+i, ((unsigned)data[3*i]*256+data[3*i+1]),data[3*i+2]);
            }
        }
    }
    else if ( (format == 0) && (radix == 2) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "Data %5d: %02x%02x\tIndex: %02x\n", init_index+i-1, data[3*i-3],data[3*i-2],data[3*i-1]);
                fprintf(fp, "Data %5d: %02x%02x\tIndex: %02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
            }
        }
    }
    else if ( (format == 0) && (radix == 3) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "Data %5d: 0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN"\tIndex: 0b"BYTETOBINARYPATTERN"\n", \
                        init_index+i-1, BYTETOBINARY(data[3*i-3]), BYTETOBINARY(data[3*i-2]), BYTETOBINARY(data[3*i-1]));
                fprintf(fp, "Data %5d: 0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN"\tIndex: 0b"BYTETOBINARYPATTERN"\n", \
                        init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
            }
        }
    }
    else if ( (format == 0) && (radix == 4) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "Data %5d: 0x%02x%02x\tIndex: 0x%02x\n", init_index+i-1, data[3*i-3],data[3*i-2],data[3*i-1]);
                fprintf(fp, "Data %5d: 0x%02x%02x\tIndex: 0x%02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
            }
        }
    }
    else if ( (format == 1) && (radix == 0) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "%5d,"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN","BYTETOBINARYPATTERN"\n", \
                        init_index+i-1, BYTETOBINARY(data[3*i-3]), BYTETOBINARY(data[3*i-2]), BYTETOBINARY(data[3*i-1]));
                fprintf(fp, "%5d,"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN","BYTETOBINARYPATTERN"\n", \
                        init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
            }
        }
    }
    else if ( (format == 1) && (radix == 1) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "%5d,%5d,%3d\n", init_index+i-1, ((unsigned)data[3*i-3]*256+data[3*i-2]),data[3*i-1]);
                fprintf(fp, "%5d,%5d,%3d\n", init_index+i, ((unsigned)data[3*i]*256+data[3*i+1]),data[3*i+2]);
            }
        }
    }
    else if ( (format == 1) && (radix == 2) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "%5d,%02x%02x,%02x\n", init_index+i-1, data[3*i-3],data[3*i-2],data[3*i-1]);
                fprintf(fp, "%5d,%02x%02x,%02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
            }
        }
    }
    else if ( (format == 1) && (radix == 3) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "%5d,0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN",0b"BYTETOBINARYPATTERN"\n", \
                        init_index+i-1, BYTETOBINARY(data[3*i-3]), BYTETOBINARY(data[3*i-2]), BYTETOBINARY(data[3*i-1]));
                fprintf(fp, "%5d,0b"BYTETOBINARYPATTERN""BYTETOBINARYPATTERN",0b"BYTETOBINARYPATTERN"\n", \
                        init_index+i, BYTETOBINARY(data[3*i]), BYTETOBINARY(data[3*i+1]), BYTETOBINARY(data[3*i+2]));
            }
        }
    }
    else if ( (format == 1) && (radix == 4) )
    {
        for (i = 1; i < sample_size; i++)
        {
            if ( (i) && ( data[3*i+2] - data[3*i-1] -1) && ((data[3*i+2]) || (data[3*i-1]-255)))
            {
                fprintf(fp,"****Info****\n");
                fprintf(fp, "%5d,0x%02x%02x,0x%02x\n", init_index+i-1, data[3*i-3],data[3*i-2],data[3*i-1]);
                fprintf(fp, "%5d,0x%02x%02x,0x%02x\n", init_index+i, data[3*i],data[3*i+1],data[3*i+2]);
            }
        }
    }
    return 0;
}

int MA_LibVer(char version[])
{
	const char libver[] =
    {
        VERSION_MAJOR_INIT,
        '.',
        VERSION_MINOR_INIT,
        '.',
        VERSION_PATCHLEVEL_INIT,
        '.',
        BUILD_YEAR_CH2, BUILD_YEAR_CH3,
        BUILD_MONTH_CH0, BUILD_MONTH_CH1,
        BUILD_DAY_CH0, BUILD_DAY_CH1,
        '.',
        BUILD_HOUR_CH0, BUILD_HOUR_CH1,
        BUILD_MIN_CH0, BUILD_MIN_CH1,
        BUILD_SEC_CH0, BUILD_SEC_CH1,
        '\0'
    };
	memcpy(version,libver,sizeof(libver));

	return 0;
}