/*!
 * \file maread.c
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 1.1
 * \date 20140108
 * \brief MA read utility.
 *
 * Copyright  2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev ?     - 20131024 - Initial version \n
 * 0.2     - rev 8     - 20131025 - Added argument parser options \n
 *                                  Updated error report format \n
 * 0.2.1   - rev       - 20131025 - Added include guard \n
 *                                  Improved debug code syntax \n
 *                                  Added doxygen style comments \n
 *                                  Changed filename \n
 * 0.2.2   - rev 23    - 20131030 - Corrected decimal format bugs \n
 * 0.3     - rev 40    - 20131104 - Changed SPI to mode 0 \n
 * 0.4     - rev 49    - 20131104 - Changed program name to "maread" \n
 *                                  Made SPI mode configurable \n
 * 0.5     - rev 64    - 20131112 - Added plain output \n
 * 0.6     - rev 87    - 20131118 - Added baudrate option \n
 * 0.7     - rev 122   - 20131211 - Corrrected data alignment error \n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 0.95    - rev 132   - 20131211 - Pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.02    - rev 143   - 20131216 - Added auto channel detection \n
 * 1.03    - rev 144   - 20121216 - Modified debug behaviour \n
 * 1.04    - rev 146   - 20121216 - Updated help messages \n
 * 1.05    - rev 148   - 20121216 - Default behaviour defined \n
 * 1.10    - rev 165   - 20140108 - Radix bug fixes \n
 * 1.9.3   - rev 210   - 20140129 - 2.0 Pre-release \n
 * \todo
 *        Add fifo depth parameter to argument
 *        Standardise error/warning messages
 */

/******************************************************************************/
/*                           Include files                                    */
/******************************************************************************/
/* Standard C libraries */
#include<stdio.h>
#include<stdlib.h>
/* OS specific libraries */
#ifdef _WIN32
#include<windows.h>
#endif

#ifndef DEBUG
#define DEBUG 0
#endif

/* Custom libraries */
#include "parser.h"
#include "ma_api.h"

#define EQUIV_FIFO_DEPTH 2500
#define MAX_FIFO_BYTESIZE EQUIV_FIFO_DEPTH*3


/******************************************************************************/
/*                      Public function definitions                                */
/******************************************************************************/


/*!
 * \brief Main function / Entry point to the sample application
 *
 * This function is the entry point to the sample application. It opens the channel, reads data
 * from the MA sensor via FTDI-USB and prints the data to the screen or to the specified file
 *
 * \param[in] argc, argv
 * \return Returns 0 for success
 * For detailed help type:
 * \code
 * $maread --help
 * \endcode
 * Example Usage:
 * \code
 * $maread -v -f output.txt -c1 -oc -rh -e -s250
 * \endcode
 * \sa
 * \note
 * \warning
 */
int main(int argc, char *argv[])
{
    uint8 *buffer; //Arbitrary memory to be malloc()'d
    FILE *outfile;
    FILE *outerror;
    int remaining_size;
    int buffer_write_count = 0;
	FT_HANDLE ftHandle;
	GEN_PARAM GenParam;
	PARSER_PARAM ParserParam;
	ChannelConfig channelConf;

    MA_AutoChannel(&GenParam);
	
	
	/*//  ==== Simple Demo Program ====
	GenParam.channel = 1;
	GenParam.muted = 1;
	GenParam.spimode = 0;
	GenParam.baudrate = 25000000;
	ParserParam.sample_size = 10;
	ParserParam.file_name[0] = '\0';
	ParserParam.output_format = 0; // 0:text 1:csv 2:plain
	ParserParam.output_radix = 2; // 0:bin 1:dec 2:hex 3:0b 4:0x
	ParserParam.error = 0;
	ParserParam.append = 0;
	printf("Opening MA\n");
	MA_Open(&ftHandle,GenParam,&channelConf);
	 buffer = (uint8 *) malloc(ParserParam.sample_size*3+3);
    if (buffer == NULL)
    {
        printf("ERROR: Could not allocate memory\n");
        exit(1);
    }
	remaining_size = ParserParam.sample_size*3+3;
	FTDI_ReadStream(ftHandle,buffer+buffer_write_count++*MAX_FIFO_BYTESIZE,remaining_size);
	MA_PrintData(stdout,buffer,0,ParserParam.sample_size,ParserParam.output_format,ParserParam.output_radix);
	/// */

	///  ==== MAread main Program ====
    // Parse command line arguments
    arg_parser(&GenParam,&ParserParam,argc,argv);

    // Initialising MA communication interface
    MA_Open(&ftHandle,GenParam,&channelConf);

    // Allocate memory
    buffer = (uint8 *) malloc(ParserParam.sample_size*3+3);
    if (buffer == NULL)
    {
        printf("ERROR: Could not allocate memory\n");
        exit(1);
    }

    // Read data
    remaining_size = ParserParam.sample_size*3;

    while (remaining_size > MAX_FIFO_BYTESIZE)
    {
        FTDI_ReadStream(ftHandle,buffer+buffer_write_count++*MAX_FIFO_BYTESIZE,MAX_FIFO_BYTESIZE);
        remaining_size -= MAX_FIFO_BYTESIZE;
    }
    MA_ReadData(ftHandle,buffer+buffer_write_count++*MAX_FIFO_BYTESIZE,remaining_size);

    // Print
    if (ParserParam.file_name[0] == '\0')
    {
        MA_PrintData(stdout,buffer,0,ParserParam.sample_size,ParserParam.output_format,ParserParam.output_radix);
    }
    else
    {
        if (ParserParam.append)
        {
            outfile = fopen(ParserParam.file_name, "a");
            outerror = fopen(strcat(ParserParam.file_name,".info"), "a");
        }
        else
        {
            outfile = fopen(ParserParam.file_name, "w");
            outerror = fopen(strcat(ParserParam.file_name,".info"), "w");
        }
            DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): File opened at %x\n",__FILE__, __LINE__, __FUNCTION__, &outfile););
            DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Error file opened at %x\n",__FILE__, __LINE__, __FUNCTION__, &outerror););
        if (outfile == NULL)
        {
            printf("ERROR: Could not open file\n");
            exit(1);
        }
        if (outerror == NULL)
        {
            printf("ERROR: Could not open error file\n");
            exit(1);
        }
        MA_PrintData(outfile,buffer,0,ParserParam.sample_size,ParserParam.output_format,ParserParam.output_radix);
        MA_PrintError(outerror,buffer,0,ParserParam.sample_size,ParserParam.output_format,ParserParam.output_radix);
        fclose(outfile);
        fclose(outerror);
    } /// */

    // Release unused resources
    free(buffer);
    MA_Close(ftHandle); // Destructor of MA_Open()

    return 0;
}

