/*!
 * \file mareg.c
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 1.1
 * \date 20140108
 * \brief MA register utility.
 *
 * Copyright  2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor register utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev 25    - 20131031 - Initial version \n
 * 0.1.1   - rev 26    - 20131031 - Update program name to fasreg \n
 * 0.2     - rev 33    - 20131031 - Updated to use SPI mode 0 \n
 * 0.3     - rev 108   - 20131209 - Hardware tested \n
 * 0.4     - rev 122   - 20131211 - Data alignment error corrected\n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 0.95    - rev 132   - 20131211 - Pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.02    - rev 143   - 20131216 - Added auto channel detection \n
 * 1.03    - rev 144   - 20121216 - Modified debug behaviour \n
 * 1.04    - rev 146   - 20121216 - Updated help messages \n
 * 1.05    - rev 148   - 20121216 - Default behaviour defined \n
 * 1.06    - rev 150   - 20131216 - Corrected bitfield write \n
 * 1.10    - rev 165   - 20140108 - Radix bug fixes \n
 * 1.9.3   - rev 210   - 20140129 - 2.0 Pre-release \n
 * \todo  Add fifo depth parameter to argument
 * \todo  Add OTP functionality
 */

/******************************************************************************/
/*                           Include files                                    */
/******************************************************************************/
/* Standard C libraries */
#include<stdio.h>
#include<stdlib.h>
/* OS specific libraries */
#ifdef _WIN32
#include<windows.h>
#endif

#ifndef DEBUG
#define DEBUG 0
#endif

/* Custom libraries */
#include "parser.h"
#include "ma_api.h"

#define EQUIV_FIFO_DEPTH 2500
#define MAX_FIFO_BYTESIZE EQUIV_FIFO_DEPTH*3

#define CHECK_WRITE(REG_NUM) \
do { if (RegParam.reg2write[REG_NUM]) \
    {MA_WriteReg_Restricted(ftHandle,REG_NUM,RegVal.regval[REG_NUM],ParserParam.key);}} \
while(0)

#define CHECK_READ(REG_NUM) \
do { if (RegParam.reg2read[REG_NUM]) \
    {MA_ReadReg(ftHandle,REG_NUM,&RegVal.regval[REG_NUM]);MA_RegPrint(RegVal.regval[REG_NUM],REG_NUM,ParserParam.output_radix);}} \
while(0)

#define CHECK_VERIFY(REG_NUM) \
do { if (RegParam.reg2write[REG_NUM]) \
    {MA_ReadReg(ftHandle,REG_NUM,&RegVal.regval[REG_NUM]);MA_RegPrint(RegVal.regval[REG_NUM],REG_NUM,ParserParam.output_radix);}} \
while(0)


/******************************************************************************/
/*                      Public function definitions                                */
/******************************************************************************/


/*!
 * \brief Main function / Entry point to the fasreg utility
 *
 * This function is the entry point to the fasreg utility. It opens the channel, reads/writes
 * registers of the MA sensor and programs the OTP.
 *
 * \param[in] argc, argv
 * \return Returns 0 for success
 * For detailed help type:
 * \code
 * $mareg --help
 * \endcode
 * Example Usage:
 * \code
 * $mareg -s25 -xh
 * \endcode
 * \sa
 * \note
 * \warning
 */
int main(int argc, char *argv[])
{
    uint8 *buffer; //Arbitrary memory to be malloc()'d
    FILE *outfile;
    FILE *outerror;
    int remaining_size;
    int buffer_write_count = 0;
	int i;
	FT_HANDLE ftHandle;
	GEN_PARAM GenParam;
	REG_PARAM RegParam;
	PARSER_PARAM ParserParam;
	ChannelConfig channelConf;
	REG_VAL RegVal;

	GenParam.channel = 1;
	GenParam.muted = 1;
	GenParam.spimode = 0;
	GenParam.baudrate = 25000000;
	ParserParam.output_radix = 0;
	ParserParam.read = 0;
	ParserParam.write = 0;
	ParserParam.verify = 0;
	ParserParam.program = 0;
	ParserParam.key = 0;

	// Initialising variables
	for (i=0;i<=9;i++)
	{
		RegVal.regval[i]=0;
		RegParam.reg2write[i]=0;
		RegParam.reg2write_msk[i]=0;
		RegParam.reg2read[i]=0;
		RegParam.reg2read_msk[i]=0;
		RegParam.reg2read_block[i]=0;
		
	}
    // Initialising MA communication interface
    if(!MA_AutoChannel(&GenParam))
    {
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Autochannel succeeded.\n",__FILE__, __LINE__, __FUNCTION__););
        MA_Open(&ftHandle,GenParam,&channelConf);
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Channel opened.\n",__FILE__, __LINE__, __FUNCTION__););
        // Read initial register values
		for (i=0;i<=9;i++)
		{
			MA_ReadReg(ftHandle,i,&RegVal.regval[i]);
		}
        MA_Close(ftHandle);
    }
    // Parse command line arguments
    arg_parser(&GenParam, &ParserParam, &RegParam, &RegVal,argc,argv);
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Arg parsed.\n",__FILE__, __LINE__, __FUNCTION__););
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): ParserParam Args %d %d %d %d.\n",__FILE__, __LINE__, __FUNCTION__,ParserParam.read,ParserParam.write,ParserParam.verify,ParserParam.program););
	DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal Args %d %d %d %d %d.\n",__FILE__, __LINE__, __FUNCTION__,RegVal.regval[0],RegVal.regval[3],RegVal.regval[4],RegVal.regval[5],RegVal.regval[7]););
    MA_Open(&ftHandle,GenParam,&channelConf);

    // Write to register
    if (ParserParam.write)
    {
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Check writes.\n",__FILE__, __LINE__, __FUNCTION__););
        for (i=0;i<=9;i++)
		{
			CHECK_WRITE(i);
			RegVal.regval[i]=0;
        }

        if (ParserParam.verify)
        {
			for (i=0;i<=9;i++)
			{
				CHECK_VERIFY(i);
            }
        }
    }

    // Read from register
    if (ParserParam.read)
    {
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Check reads.\n",__FILE__, __LINE__, __FUNCTION__););
		for (i=0;i<=9;i++)
		{
			CHECK_READ(i);
        }
    }

    // Release unused resources

    MA_Close(ftHandle);

    return 0;
}

