/*!
 * \file parser.c
 * \author Benjamin Zhai (benjamin.zhai@sensima.com)
 * \version 1.1
 * \date 20140108
 * \brief Main program argument parser
 *
 * Copyright © 2013-2014 Sensima Technology SA \n
 * Company Confidential \n
 *
 * Project: MA-FTDI USB \n
 * Module: MA sensor communication utility using FTDI USB \n
 *
 * Revision History: \n
 * 0.1     - rev ?     - 20131024 - Initial version \n
 * 0.2     - rev 8     - 20131025 - Added argument parser options \n
 *                                  Updated error report format \n
 * 0.2.1   - rev       - 20131025 - Added include guard \n
 *                                  Improved debug code syntax \n
 *                                  Added doxygen style comments \n
 *                                  Changed filename \n
 * 0.2.2   - rev 23    - 20131030 - Corrected decimal format bugs \n
 * 0.3     - rev 40    - 20131104 - Changed SPI to mode 0 \n
 * 0.4     - rev 49    - 20131104 - Changed program name to "maread" \n
 *                                  Made SPI mode configurable \n
 * 0.5     - rev 64    - 20131112 - Added plain output \n
 * 0.6     - rev 87    - 20131118 - Added baudrate option \n
 * 0.9     - rev 123   - 20131211 - Merged pre-release\n
 * 1.0     - rev 136   - 20131212 - Release \n
 * 1.02    - rev 143   - 20131216 - Added auto channel detection \n
 * 1.03    - rev 144   - 20121216 - Modified debug behaviour \n
 * 1.04    - rev 146   - 20121216 - Updated help messages \n
 * 1.05    - rev 148   - 20121216 - Default behaviour defined \n
 * 1.10    - rev 165   - 20140108 - Radix bug fixes \n
 * 1.9.3   - rev 210   - 20140129 - 2.0 Pre-release \n
 * \todo
 */
#include "parser.h"

int arg_parser(GEN_PARAM* GenParam, PARSER_PARAM* ParserParam, REG_PARAM* RegParam, REG_VAL* RegVal, int argc, char *argv[])
{
    int arg_parsed = 0;
    int exit_now = 0;
    char *str1;
	char malib_ver[30];
    unsigned char completeVersion[] =
    {
        VERSION_MAJOR_INIT,
        '.',
        VERSION_MINOR_INIT,
        '.',
        VERSION_PATCHLEVEL_INIT,
        '.',
        BUILD_YEAR_CH2, BUILD_YEAR_CH3,
        BUILD_MONTH_CH0, BUILD_MONTH_CH1,
        BUILD_DAY_CH0, BUILD_DAY_CH1,
        '.',
        BUILD_HOUR_CH0, BUILD_HOUR_CH1,
        BUILD_MIN_CH0, BUILD_MIN_CH1,
        BUILD_SEC_CH0, BUILD_SEC_CH1,
        '\0'
    };
	ParserParam->output_radix = 0; // 0:bin 1:dec 2:hex 3:0b 4:0x
	ParserParam->read = 0;
	ParserParam->write = 0;
	ParserParam->verify = 0;
	ParserParam->program = 0;

    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Program name: %s\n",__FILE__, __LINE__, __FUNCTION__, argv[0]););
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Number of arguments: %d\n",__FILE__, __LINE__, __FUNCTION__, argc););

    // set default behaviour
    if (argc ==1)
    {
        ParserParam->read = 1;
        RegParam->reg2read[0] = 1;
        RegParam->reg2read[3] = 1;
        RegParam->reg2read[4] = 1;
        RegParam->reg2read[5] = 1;
        RegParam->reg2read[7] = 1;
    }
    arg_parsed++;
    while ( (argc > arg_parsed) && (!exit_now))
    {
        if ( (argv[arg_parsed][0] == '-') || (argv[arg_parsed][0] == '/'))
        {
            switch (argv[arg_parsed][1])
            {
                case '-':
                    if (!strcmp(argv[arg_parsed],"--help"))
                        argv[arg_parsed][1] = 'h';
                    else if (!strcmp(argv[arg_parsed],"--version"))
                        argv[arg_parsed][1] = 'v';
                    else if (!strcmp(argv[arg_parsed],"--list"))
                        argv[arg_parsed][1] = 'l';
                    else if (!strcmp(argv[arg_parsed],"--read"))
                        argv[arg_parsed][1] = 'r';
                    else if (!strcmp(argv[arg_parsed],"--write"))
                        argv[arg_parsed][1] = 'w';
                    else if (!strcmp(argv[arg_parsed],"--OTP"))
                        argv[arg_parsed][1] = '!';
                    else if (!strcmp(argv[arg_parsed],"--verify"))
                        argv[arg_parsed][1] = 'x';
                    else if (!strcmp(argv[arg_parsed],"--radix-bin"))
                        argv[arg_parsed] = "-xb";
                    else if (!strcmp(argv[arg_parsed],"--radix-dec"))
                        argv[arg_parsed] = "-xd";
                    else if (!strcmp(argv[arg_parsed],"--radix-hex"))
                        argv[arg_parsed] = "-xh";
                    else if (!strcmp(argv[arg_parsed],"--radix-bin-prefix"))
                        argv[arg_parsed] = "-x0b";
                    else if (!strcmp(argv[arg_parsed],"--radix-hex-prefix"))
                        argv[arg_parsed] = "-x0x";
					else if (!strcmp(argv[arg_parsed],"--r+"))
                    {
						ParserParam->read = 1;
						RegParam->reg2read[0] = 1;
						RegParam->reg2read[1] = 1;
						RegParam->reg2read[2] = 1;
						RegParam->reg2read[3] = 1;
						RegParam->reg2read[4] = 1;
						RegParam->reg2read[5] = 1;
						RegParam->reg2read[6] = 1;
						RegParam->reg2read[7] = 1;
						RegParam->reg2read[8] = 1;
						arg_parsed++;
					}
                    else
                    {
                        printf("Syntax error: Invalid argument: %s. For help type:\n%s --help\n", argv[arg_parsed++],argv[0]);
                        #ifndef DEBUG
                            exit(1);
                        #endif
                    }
                    break;
                case 'h':
                    printf( "Usage: %s [options] ...\n"\
                            "-l [--list]\t\t\tLists channel information\n"\
                            "-x {RADIX} \t\t\tSets radix to RADIX. RADIX=bin,dec,hex,0b,0x\n"\
                            "  -xb [--radix-bin]\t\t  Sets radix to binary\n"\
                            "  -xd [--radix-dec]\t\t  Sets radix to decimal\n"\
                            "  -xh [--radix-hex]\t\t  Sets radix to hex\n"\
                            "  -x0b [--radix-bin-prefix]\t  Sets radix to binary with '0b' prefix\n"\
                            "  -x0x [--radix-hex-prefix]\t  Sets radix to hex with '0x' prefix\n"\
                            "-r [--read] \t\t\tReads registers specified\n"\
                            "-w [--write] \t\t\tWrites registers specified\n"\
                            "-y [--verify] \t\t\tVerifies registers written\n"\
                            "-h [--help]\t\t\tDisplays this help\n"\
                            "-v [--version]\t\t\tDisplays version information\n"\
                            "This program is built for "SYSTEM_ARCH" with gcc "__VERSION__"\n"\
                            "Report bugs to <benjamin.zhai@sensima.com>\n",argv[0]);
                    arg_parsed++;
                    exit_now = 1;
                    break;
                case 'v':
					MA_LibVer(malib_ver);
                    printf( "MagAlpha register utility v%s\n"\
                            "==================================================\n"\
                            "MA Utility API version %s\n"\
                            "Copyright (C) "YEARS" Sensima Technology SA.\n"\
                            "All rights reserved.\n"\
                            "This software is provided AS IS with NO warranty.\n",completeVersion,malib_ver);
                    arg_parsed++;
                    exit_now = 1;
                    break;
                case 'x':
                    if (argv[arg_parsed][2] == '\0')
                    {
                        DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -x: Output radix argument is %s\n",argv[arg_parsed+1]););
                        ParserParam->output_radix |= strcmp(argv[++arg_parsed],"dec")? 0:1;
                        ParserParam->output_radix |= strcmp(argv[arg_parsed],"hex")? 0:2;
                        ParserParam->output_radix |= strcmp(argv[arg_parsed],"0b")? 0:3;
                        ParserParam->output_radix |= strcmp(argv[arg_parsed++],"0x")? 0:4;
                        DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -x: Output radix is %d\n",ParserParam->output_radix););
                    }
                    else
                    {
                        DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -x?: Output radix argument is %s\n",argv[arg_parsed]+2););
                        ParserParam->output_radix |= strcmp(argv[arg_parsed]+2,"d")? 0:1;
                        ParserParam->output_radix |= strcmp(argv[arg_parsed]+2,"h")? 0:2;
                        ParserParam->output_radix |= strcmp(argv[arg_parsed]+2,"0b")? 0:3;
                        ParserParam->output_radix |= strcmp(argv[arg_parsed++]+2,"0x")? 0:4;
                        DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -x?: Output radix is %d\n",ParserParam->output_radix););
                    }
                    break;
                case 'p':
                    GenParam->spimode = (int) (strtol(argv[arg_parsed++]+2,NULL,10)%4);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -p: SPI mode is %d\n",GenParam->spimode););
                    break;
                case 'b':
                    GenParam->baudrate = (int) strtol(argv[arg_parsed++]+2,NULL,10);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -b: Baudrate is %d\n",GenParam->baudrate););
                    break;
                case 'r':
                    ParserParam->read = 1;
                    arg_parsed++;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -r: Read\n"););
                    break;
                case 'l':
                    GenParam->muted = 0;
                    arg_parsed++;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -l: List devices\n"););
                    break;
                case 'w':
                    ParserParam->write = 1;
                    arg_parsed++;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -w: Write\n"););
                    break;
                case '!':
                    ParserParam->program = 1;
                    arg_parsed++;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -!: Flash\n"););
                    break;
                case 'y':
                    ParserParam->verify = 1;
                    arg_parsed++;
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -y: Verify\n"););
                    break;
				case 'k':
					if (argv[arg_parsed][2] == '=')
						ParserParam->key = (int) strtol(argv[arg_parsed++]+3,NULL,0);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: -k: Key %x\n",ParserParam->key););
                    break;
                default:
                    printf("Syntax error: Invalid argument: %s. For help type:\n%s --help\n", argv[arg_parsed++],argv[0]);
                    #ifndef DEBUG
                        exit(1);
                    #endif
                    break;
            }
        }
        else
        {
            DEBUG_CODE(fprintf(stderr,"***DEBUG: Argument parser: Registers\n"););
            if (parser_regset(*ParserParam,RegParam,RegVal,argv[arg_parsed++])) // parse register actions
            {
                printf("Syntax error: Invalid argument: %s. For help type:\n%s --help\n", argv[arg_parsed-1],argv[0]);
                #ifndef DEBUG
                    exit(1);
                #endif
            }
        }
    }
    if (exit_now)
        exit(0);
    return 0;
}


// Register action parsing function
int parser_regset(PARSER_PARAM ParserParam, REG_PARAM* RegParam, REG_VAL* RegVal, char* argstr)
{
    int retval = 0;
	int i;
    char *str1;
    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Current argument: %s\n",__FILE__, __LINE__, __FUNCTION__,argstr););
	
    str1 = (char*) strchr(argstr,'=');
    if ((str1 == NULL)&&(ParserParam.read)) // read
    {
        if(isdigit(argstr[0])) // if block read
        {
            RegParam->reg2read[argstr[0] - '0'] = 1;
            RegParam->reg2read_msk[argstr[0] - '0'] = 0xff;
            RegParam->reg2read_block[argstr[0] - '0'] = 1;
        }
        else // if addressed by bitfield
        {
            if (!strcmp(argstr,"EDF"))
            {
                RegParam->reg2read[0] = 1;
                RegParam->reg2read_msk[0] |= 0x01;
            }
            else if (!strcmp(argstr,"FWS"))
            {
                RegParam->reg2read[0] = 1;
                RegParam->reg2read_msk[0] |= 0x02;
            }
            else if (!strcmp(argstr,"FR"))
            {
                RegParam->reg2read[0] = 1;
                RegParam->reg2read_msk[0] |= 0x04;
            }
            else if (!strcmp(argstr,"FLL"))
            {
                RegParam->reg2read[0] = 1;
                RegParam->reg2read_msk[0] |= 0x18;
            }
            else if (!strcmp(argstr,"FI"))
            {
                RegParam->reg2read[0] = 1;
                RegParam->reg2read_msk[0] |= 0xe0;
            }
            else if (!strcmp(argstr,"BCT"))
            {
                RegParam->reg2read[3] = 1;
                RegParam->reg2read_msk[3] |= 0xff;
            }
            else if (!strcmp(argstr,"ZH"))
            {
                RegParam->reg2read[4] = 1;
                RegParam->reg2read_msk[4] |= 0xff;
            }
            else if (!strcmp(argstr,"ZL"))
            {
                RegParam->reg2read[5] = 1;
                RegParam->reg2read_msk[5] |= 0x0f;
            }
            else if (!strcmp(argstr,"ETX"))
            {
                RegParam->reg2read[5] = 1;
                RegParam->reg2read_msk[5] |= 0x10;
            }
            else if (!strcmp(argstr,"ETY"))
            {
                RegParam->reg2read[5] = 1;
                RegParam->reg2read_msk[5] |= 0x20;
            }
            else if (!strcmp(argstr,"BIX"))
            {
                RegParam->reg2read[5] = 1;
                RegParam->reg2read_msk[5] |= 0x40;
            }
            else if (!strcmp(argstr,"BIY"))
            {
                RegParam->reg2read[5] = 1;
                RegParam->reg2read_msk[5] |= 0x80;
            }
            else if (!strcmp(argstr,"PWM"))
            {
                RegParam->reg2read[7] = 1;
                RegParam->reg2read_msk[7] |= 0x01;
            }
            else if (!strcmp(argstr,"RD"))
            {
                RegParam->reg2read[7] = 1;
                RegParam->reg2read_msk[7] |= 0x02;
            }
            else if (!strcmp(argstr,"UVW"))
            {
                RegParam->reg2read[7] = 1;
                RegParam->reg2read_msk[7] |= 0x04;
            }
            else if (!strcmp(argstr,"ABZ"))
            {
                RegParam->reg2read[7] = 1;
                RegParam->reg2read_msk[7] |= 0x08;
            }
            else if (!strcmp(argstr,"EPT"))
            {
                RegParam->reg2read[7] = 1;
                RegParam->reg2read_msk[7] |= 0x30;
            }
            else if (!strcmp(argstr,"AF"))
            {
                RegParam->reg2read[7] = 1;
                RegParam->reg2read_msk[7] |= 0xc0;
            }
        }
    }
    else if ((str1 != NULL)&&(ParserParam.write)) // write
    {
        *str1++ = '\0'; // split into two strings
        DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Parameter before '=': %s\n",__FILE__, __LINE__, __FUNCTION__, argstr););
        DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Parameter after '=': %s\n",__FILE__, __LINE__, __FUNCTION__, str1););
        if(isdigit(argstr[0])) // starting with number -> register number
        {
            RegParam->reg2write[argstr[0] - '0'] = 1;
            RegParam->reg2write_msk[argstr[0] - '0'] = 0xff;
            switch(argstr[0] - '0')
            {
                case 0:
				case 1:
				case 2:
                case 3:
                case 4:
                case 5:
				case 6:
                case 7:
				case 8:
                    RegVal->regval[argstr[0] - '0'] = (uint8) strtol(str1,NULL,0);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[%d]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, argstr[0] - '0', RegVal->regval[argstr[0] - '0']););
                    break;
                case 9: // program directly
                    if(ParserParam.program)
                        RegVal->regval[9] = (uint8) strtol(str1,NULL,0);
                    DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[9]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[9]););
                    break;
            }
        }
        else // not starting with number -> bit field name
        {
            if (!strcmp(argstr,"EDF"))
            {
                RegParam->reg2write[0] = 1;
                RegParam->reg2write_msk[0] |= 0x01;
                RegVal->regval[0] &= 0xfe;
                RegVal->regval[0] |= ((uint8) strtol(str1,NULL,0)&0x01);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[0]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[0]););
            }
            else if (!strcmp(argstr,"FWS"))
            {
                RegParam->reg2write[0] = 1;
                RegParam->reg2write_msk[0] |= 0x02;
                RegVal->regval[0] &= 0xfd;
                RegVal->regval[0] |= ((uint8) strtol(str1,NULL,0)<<1&0x02);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[0]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[0]););
            }
            else if (!strcmp(argstr,"FR"))
            {
                RegParam->reg2write[0] = 1;
                RegParam->reg2write_msk[0] |= 0x04;
                RegVal->regval[0] &= 0xfb;
                RegVal->regval[0] |= ((uint8) strtol(str1,NULL,0)<<2&0x04);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[0]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[0]););
            }
            else if (!strcmp(argstr,"FLL"))
            {
                RegParam->reg2write[0] = 1;
                RegParam->reg2write_msk[0] |= 0x18;
                RegVal->regval[0] &= 0xe7;
                RegVal->regval[0] |= ((uint8) strtol(str1,NULL,0)<<3&0x18);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[0]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[0]););
            }
            else if (!strcmp(argstr,"FI"))
            {
                RegParam->reg2write[0] = 1;
                RegParam->reg2write_msk[0] |= 0xe0;
                RegVal->regval[0] &= 0x1f;
                RegVal->regval[0] |= ((uint8) strtol(str1,NULL,0)<<5&0xe0);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[0]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[0]););
            }
            else if (!strcmp(argstr,"BCT"))
            {
                RegParam->reg2write[3] = 1;
                RegParam->reg2write_msk[3] |= 0xff;
                RegVal->regval[3] &= 0x00;
                RegVal->regval[3] |= ((uint8) strtol(str1,NULL,0)&0xff);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[3]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[3]););
            }
            else if (!strcmp(argstr,"ZH"))
            {
                RegParam->reg2write[4] = 1;
                RegParam->reg2write_msk[4] |= 0xff;
                RegVal->regval[4] &= 0x00;
                RegVal->regval[4] |= ((uint8) strtol(str1,NULL,0)&0xff);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[4]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[4]););
            }
            else if (!strcmp(argstr,"ZL"))
            {
                RegParam->reg2write[5] = 1;
                RegParam->reg2write_msk[5] |= 0x0f;
                RegVal->regval[5] &= 0xf0;
                RegVal->regval[5] |= ((uint8) strtol(str1,NULL,0)&0x0f);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[5]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[5]););
            }
            else if (!strcmp(argstr,"ETX"))
            {
                RegParam->reg2write[5] = 1;
                RegParam->reg2write_msk[5] |= 0x10;
                RegVal->regval[5] &= 0xef;
                RegVal->regval[5] |= ((uint8) strtol(str1,NULL,0)<<4&0x10);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[5]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[5]););
            }
            else if (!strcmp(argstr,"ETY"))
            {
                RegParam->reg2write[5] = 1;
                RegParam->reg2write_msk[5] |= 0x20;
                RegVal->regval[5] &= 0xdf;
                RegVal->regval[5] |= ((uint8) strtol(str1,NULL,0)<<5&0x20);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[5]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[5]););
            }
            else if (!strcmp(argstr,"BIX"))
            {
                RegParam->reg2write[5] = 1;
                RegParam->reg2write_msk[5] |= 0x40;
                RegVal->regval[5] &= 0xbf;
                RegVal->regval[5] |= ((uint8) strtol(str1,NULL,0)<<6&0x40);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[5]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[5]););
            }
            else if (!strcmp(argstr,"BIY"))
            {
                RegParam->reg2write[5] = 1;
                RegParam->reg2write_msk[5] |= 0x80;
                RegVal->regval[5] &= 0x7f;
                RegVal->regval[5] |= ((uint8) strtol(str1,NULL,0)<<7&0x80);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[5]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[5]););
            }
            else if (!strcmp(argstr,"PWM"))
            {
                RegParam->reg2write[7] = 1;
                RegParam->reg2write_msk[7] |= 0x01;
                RegVal->regval[7] &= 0xfe;
                RegVal->regval[7] |= ((uint8) strtol(str1,NULL,0)&0x01);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[7]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[7]););
            }
            else if (!strcmp(argstr,"RD"))
            {
                RegParam->reg2write[7] = 1;
                RegParam->reg2write_msk[7] |= 0x02;
                RegVal->regval[7] &= 0xfd;
                RegVal->regval[7] |= ((uint8) strtol(str1,NULL,0)<<1&0x02);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[7]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[7]););
            }
            else if (!strcmp(argstr,"UVW"))
            {
                RegParam->reg2write[7] = 1;
                RegParam->reg2write_msk[7] |= 0x04;
                RegVal->regval[7] &= 0xfb;
                RegVal->regval[7] |= ((uint8) strtol(str1,NULL,0)<<2&0x04);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[7]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[7]););
            }
            else if (!strcmp(argstr,"ABZ"))
            {
                RegParam->reg2write[7] = 1;
                RegParam->reg2write_msk[7] |= 0x08;
                RegVal->regval[7] &= 0xf7;
                RegVal->regval[7] |= ((uint8) strtol(str1,NULL,0)<<3&0x08);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[7]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[7]););
            }
            else if (!strcmp(argstr,"EPT"))
            {
                RegParam->reg2write[7] = 1;
                RegParam->reg2write_msk[7] |= 0x30;
                RegVal->regval[7] &= 0xcf;
                RegVal->regval[7] |= ((uint8) strtol(str1,NULL,0)<<4&0x30);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[7]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[7]););
            }
            else if (!strcmp(argstr,"AF"))
            {
                RegParam->reg2write[7] = 1;
                RegParam->reg2write_msk[7] |= 0xc0;
                RegVal->regval[7] &= 0x3f;
                RegVal->regval[7] |= ((uint8) strtol(str1,NULL,0)<<6&0xc0);
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[7]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[7]););
            }
			else if ( (argstr[0]=='R') && (argstr[2]=='B') )
            {
                RegParam->reg2write[argstr[1]-'0'] = 1;
                RegParam->reg2write_msk[argstr[1]-'0'] |= (1<<(argstr[3]-'0'));
                RegVal->regval[argstr[1]-'0'] &= (0xff - (1<<(argstr[3]-'0')));
                RegVal->regval[argstr[1]-'0'] |= ((uint8) strtol(str1,NULL,0)<<(argstr[3]-'0')&(1<<(argstr[3]-'0')));
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[%d]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, argstr[1]-'0', RegVal->regval[argstr[1]-'0']););
            }
        }
    }
	else if ((str1 == NULL)&&(ParserParam.program)) // program
	{
		if ((argstr[0]=='F') && (isdigit(argstr[1])))
            {
                RegParam->reg2write[9] = 1;
                RegParam->reg2write_msk[9] |= (1<<(argstr[1]-'0'));
                RegVal->regval[9] = (1<<(argstr[1]-'0'));
                DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): RegVal->regval[9]: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, RegVal->regval[9]););
            }
	}
    else
    {
        retval = 1;
    }

    // debug output of register configurations
	for (i=0;i<=9;i++)
	{
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Reg%dW: %d Val: 0x%02x Msk: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, i, RegParam->reg2write[i],RegVal->regval[i],RegParam->reg2write_msk[i]););
		DEBUG_CODE(fprintf(stderr,"***DEBUG: %s:%d:%s(): Reg%dR: %d Msk: 0x%02x\n",__FILE__, __LINE__, __FUNCTION__, i, RegParam->reg2read[i],RegParam->reg2read_msk[i]););
	}
    return retval;
}
